@@ -65,7 +65,7 @@ impl Default for KeymapConfig {
                 },
                 Keymap {
                     key_sequence: "/".into(),
-                    command: Command::SearchContext,
+                    command: Command::Search,
                 },
                 Keymap {
                     key_sequence: "C-space".into(),
@@ -111,6 +111,10 @@ impl Default for KeymapConfig {
                     key_sequence: "g space".into(),
                     command: Command::BrowsePlayingContext,
                 },
+                Keymap {
+                    key_sequence: "g l".into(),
+                    command: Command::LibraryPage,
+                },
                 Keymap {
                     key_sequence: "g s".into(),
                     command: Command::SearchPage,
@@ -202,8 +206,8 @@ impl KeymapConfig {
     pub fn parse_config_file(&mut self, path: &std::path::Path) -> Result<()> {
         match std::fs::read_to_string(path.join(super::KEYMAP_CONFIG_FILE)) {
             Err(err) => {
-                log::warn!(
-                    "failed to open the keymap config file: {:#?}...\nUse the default configurations instead...",
+                tracing::warn!(
+                    "failed to open the keymap config file: {:?}...\nUse the default configurations instead...",
                     err
                 );
             }
