@@ -7,7 +7,6 @@ use crate::{
 use anyhow::Result;
 use crossterm::event::*;
 use rand::Rng;
-use rspotify::model;
 use std::sync::mpsc;
 use tokio_stream::StreamExt;
 
@@ -24,6 +23,7 @@ pub enum PlayerRequest {
     Repeat,
     Shuffle,
     Volume(u8),
+    Reconnect,
     TransferPlayback(String, bool),
     StartPlayback(Playback),
 }
@@ -46,24 +46,25 @@ pub enum ClientRequest {
 }
 
 #[tokio::main]
-/// starts a handler to handle terminal events (key pressed, mouse clicked, etc)
+/// starts a terminal event handler (key pressed, mouse clicked, etc)
 pub async fn start_event_handler(send: mpsc::Sender<ClientRequest>, state: SharedState) {
     let mut event_stream = EventStream::new();
 
     while let Some(event) = event_stream.next().await {
         match event {
             Ok(event) => {
-                log::info!("got event: {:?}", event);
+                tracing::info!("got a terminal event: {:?}", event);
+
                 if let Err(err) = match event {
                     Event::Mouse(event) => handle_mouse_event(event, &send, &state),
                     Event::Key(event) => handle_key_event(event, &send, &state),
                     _ => Ok(()),
                 } {
-                    log::warn!("failed to handle event: {:#}", err);
+                    tracing::warn!("failed to handle event: {}", err);
                 }
             }
             Err(err) => {
-                log::warn!("failed to get event: {:#}", err);
+                tracing::warn!("failed to get event: {}", err);
             }
         }
     }
@@ -75,11 +76,14 @@ fn handle_mouse_event(
     send: &mpsc::Sender<ClientRequest>,
     state: &SharedState,
 ) -> Result<()> {
-    let ui = state.ui.lock().unwrap();
+    let ui = state.ui.lock();
     // a left click event
     if let MouseEventKind::Down(MouseButton::Left) = event.kind {
         if event.row == ui.progress_bar_rect.y {
-            let player = state.player.read().unwrap();
+            // calculate the seek position (in ms) based on the clicked position,
+            // the progress bar's width and the track's duration (in ms)
+
+            let player = state.player.read();
             let track = player.current_playing_track();
             if let Some(track) = track {
                 let position_ms = (track.duration.as_millis() as u32) * (event.column as u32)
@@ -99,12 +103,8 @@ fn handle_key_event(
 ) -> Result<()> {
     let key: Key = event.into();
 
-    // lock the UI state because we'll use it many times
-    // when handling a terminal event
-    let mut ui = state.ui.lock().unwrap();
-
     // parse the key sequence from user's previous inputs
-    let mut key_sequence = ui.input_key_sequence.clone();
+    let mut key_sequence = state.ui.lock().input_key_sequence.clone();
     key_sequence.keys.push(key.clone());
     if state
         .keymap_config
@@ -114,35 +114,37 @@ fn handle_key_event(
         key_sequence = KeySequence { keys: vec![key] };
     }
 
+    let ui = state.ui.lock();
     let handled = match ui.popup {
         None => {
             // no popup
             match ui.current_page() {
+                PageState::Library => {
+                    drop(ui);
+                    window::handle_key_sequence_for_library_window(&key_sequence, state)?
+                }
                 PageState::Recommendations(..) => {
+                    drop(ui);
                     window::handle_key_sequence_for_recommendation_window(
                         &key_sequence,
                         send,
                         state,
-                        &mut ui,
                     )?
                 }
-                PageState::Browsing(_) | PageState::CurrentPlaying => {
-                    window::handle_key_sequence_for_context_window(
-                        &key_sequence,
-                        send,
-                        state,
-                        &mut ui,
-                    )?
+                PageState::Context(..) => {
+                    drop(ui);
+                    window::handle_key_sequence_for_context_window(&key_sequence, send, state)?
+                }
+                PageState::Searching { .. } => {
+                    drop(ui);
+                    window::handle_key_sequence_for_search_window(&key_sequence, send, state)?
                 }
-                PageState::Searching(..) => window::handle_key_sequence_for_search_window(
-                    &key_sequence,
-                    send,
-                    state,
-                    &mut ui,
-                )?,
             }
         }
-        Some(_) => popup::handle_key_sequence_for_popup(&key_sequence, send, state, &mut ui)?,
+        Some(_) => {
+            drop(ui);
+            popup::handle_key_sequence_for_popup(&key_sequence, send, state)?
+        }
     };
 
     // if the key sequence is not handled, let the global command handler handle it
@@ -151,7 +153,7 @@ fn handle_key_event(
             .keymap_config
             .find_command_from_key_sequence(&key_sequence)
         {
-            Some(command) => handle_global_command(command, send, state, &mut ui)?,
+            Some(command) => handle_global_command(command, send, state)?,
             None => false,
         }
     } else {
@@ -161,9 +163,9 @@ fn handle_key_event(
     // if successfully handled the key sequence, clear the key sequence.
     // else, the current key sequence is probably a prefix of a command's shortcut
     if handled {
-        ui.input_key_sequence.keys = vec![];
+        state.ui.lock().input_key_sequence.keys = vec![];
     } else {
-        ui.input_key_sequence = key_sequence;
+        state.ui.lock().input_key_sequence = key_sequence;
     }
     Ok(())
 }
@@ -173,8 +175,9 @@ fn handle_global_command(
     command: Command,
     send: &mpsc::Sender<ClientRequest>,
     state: &SharedState,
-    ui: &mut UIStateGuard,
 ) -> Result<bool> {
+    let mut ui = state.ui.lock();
+
     match command {
         Command::Quit => {
             ui.is_running = false;
@@ -195,7 +198,7 @@ fn handle_global_command(
             send.send(ClientRequest::Player(PlayerRequest::Shuffle))?;
         }
         Command::VolumeUp => {
-            if let Some(ref playback) = state.player.read().unwrap().playback {
+            if let Some(ref playback) = state.player.read().playback {
                 if let Some(percent) = playback.device.volume_percent {
                     let volume = std::cmp::min(percent + 5, 100_u32);
                     send.send(ClientRequest::Player(PlayerRequest::Volume(volume as u8)))?;
@@ -203,7 +206,7 @@ fn handle_global_command(
             }
         }
         Command::VolumeDown => {
-            if let Some(ref playback) = state.player.read().unwrap().playback {
+            if let Some(ref playback) = state.player.read().playback {
                 if let Some(percent) = playback.device.volume_percent {
                     let volume = std::cmp::max(percent.saturating_sub(5_u32), 0_u32);
                     send.send(ClientRequest::Player(PlayerRequest::Volume(volume as u8)))?;
@@ -211,13 +214,13 @@ fn handle_global_command(
             }
         }
         Command::OpenCommandHelp => {
-            ui.popup = Some(PopupState::CommandHelp(0));
+            ui.popup = Some(PopupState::CommandHelp { offset: 0 });
         }
         Command::RefreshPlayback => {
             send.send(ClientRequest::GetCurrentPlayback)?;
         }
         Command::ShowActionsOnCurrentTrack => {
-            if let Some(track) = state.player.read().unwrap().current_playing_track() {
+            if let Some(track) = state.player.read().current_playing_track() {
                 ui.popup = Some(PopupState::ActionList(
                     Item::Track(track.clone().into()),
                     new_list_state(),
@@ -225,13 +228,12 @@ fn handle_global_command(
             }
         }
         Command::BrowsePlayingContext => {
-            ui.new_page(PageState::CurrentPlaying);
+            ui.create_new_page(PageState::Context(None, ContextPageType::CurrentPlaying));
         }
         Command::BrowseUserPlaylists => {
             send.send(ClientRequest::GetUserPlaylists)?;
             ui.popup = Some(PopupState::UserPlaylistList(
                 PlaylistPopupAction::Browse,
-                state.player.read().unwrap().user_playlists.to_vec(),
                 new_list_state(),
             ));
         }
@@ -243,24 +245,26 @@ fn handle_global_command(
             send.send(ClientRequest::GetUserSavedAlbums)?;
             ui.popup = Some(PopupState::UserSavedAlbumList(new_list_state()));
         }
+        Command::LibraryPage => {
+            ui.create_new_page(PageState::Library);
+        }
         Command::SearchPage => {
-            ui.new_page(PageState::Searching(
-                "".to_owned(),
-                Box::new(SearchResults::default()),
-            ));
-            ui.window = WindowState::Search(
-                new_list_state(),
-                new_list_state(),
-                new_list_state(),
-                new_list_state(),
-                SearchFocusState::Input,
-            );
+            ui.create_new_page(PageState::Searching {
+                input: "".to_owned(),
+                current_query: "".to_owned(),
+            });
         }
         Command::PreviousPage => {
             if ui.history.len() > 1 {
                 ui.history.pop();
                 ui.popup = None;
                 ui.window = WindowState::Unknown;
+
+                // empty the previous page's `context_id` to force
+                // updating the context page's window state and requesting the context data
+                if let PageState::Context(ref mut context_id, _) = ui.current_page_mut() {
+                    *context_id = None;
+                }
             }
         }
         Command::SwitchDevice => {
@@ -268,7 +272,15 @@ fn handle_global_command(
             send.send(ClientRequest::GetDevices)?;
         }
         Command::SwitchTheme => {
-            ui.popup = Some(PopupState::ThemeList(state.themes(ui), new_list_state()));
+            // get the available themes with the current theme moved to the first position
+            let mut themes = state.theme_config.themes.clone();
+            let id = themes.iter().position(|t| t.name == ui.theme.name);
+            if let Some(id) = id {
+                let theme = themes.remove(id);
+                themes.insert(0, theme);
+            }
+
+            ui.popup = Some(PopupState::ThemeList(themes, new_list_state()));
         }
         _ => return Ok(false),
     }
